import { Path, FilterDefinitions } from '@supabase-cache-helpers/postgrest-filter';
import { PostgrestQueryBuilder, PostgrestTransformBuilder } from '@supabase/postgrest-js';
import { GetResult } from '@supabase/postgrest-js/dist/module/select-query-parser';
import { GenericTable, GenericSchema } from '@supabase/postgrest-js/dist/module/types';
import { AnyPostgrestResponse, PostgrestPaginationResponse, PostgrestHasMorePaginationResponse } from '@supabase-cache-helpers/postgrest-shared';
import { GenericSchema as GenericSchema$1 } from '@supabase/supabase-js/dist/module/lib/types';

type LoadQueryOps<Q extends string = '*'> = {
    query?: (Q extends '*' ? "'*' is not allowed" : Q) | null;
    disabled?: boolean;
    queriesForTable: () => {
        paths: Path[];
        filters: FilterDefinitions;
    }[];
};
type LoadQueryReturn = {
    selectQuery: string;
    userQueryPaths: Path[] | null;
    paths: Path[];
};
declare const loadQuery: <Q extends string = "*">({ query, disabled, queriesForTable, }: LoadQueryOps<Q>) => LoadQueryReturn | null;

type MutationFetcherResponse<R> = {
    normalizedData: R;
    userQueryData?: R;
};

type DeleteFetcher<T extends GenericTable, R> = (input: Partial<T['Row']>) => Promise<MutationFetcherResponse<R> | null>;
declare const buildDeleteFetcher: <S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T["Row"])[], opts: LoadQueryOps<Q>) => DeleteFetcher<T, R>;

declare const fetcher: <Result>(q: PromiseLike<AnyPostgrestResponse<Result>>) => Promise<AnyPostgrestResponse<Result>>;

type InsertFetcher<T extends GenericTable, R> = (input: T['Insert'][]) => Promise<MutationFetcherResponse<R>[] | null>;
declare function buildInsertFetcher<S extends GenericSchema$1, T extends GenericTable, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, opts: LoadQueryOps<Q>): InsertFetcher<T, R>;

type PostgrestPaginationFetcher<Type, Args> = (args: Args) => Promise<Type>;
type PostgrestPaginationKeyDecoder<Args> = (args: Args) => {
    limit?: number;
    offset?: number;
};
declare const createPaginationFetcher: <Schema extends GenericSchema$1, Row extends Record<string, unknown>, Result, Args>(query: PostgrestTransformBuilder<Schema, Row, Result[]> | null, decode: PostgrestPaginationKeyDecoder<Args>, pageSize: number) => PostgrestPaginationFetcher<PostgrestPaginationResponse<Result>, Args> | null;
declare const createPaginationHasMoreFetcher: <Schema extends GenericSchema$1, Row extends Record<string, unknown>, Result, Args>(query: PostgrestTransformBuilder<Schema, Row, Result[]> | null, decode: PostgrestPaginationKeyDecoder<Args>, pageSize: number) => PostgrestPaginationFetcher<PostgrestHasMorePaginationResponse<Result>, Args> | null;

type UpdateFetcher<T extends GenericTable, R> = (input: Partial<T['Row']>) => Promise<MutationFetcherResponse<R> | null>;
declare const buildUpdateFetcher: <S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T["Row"])[], opts: LoadQueryOps<Q>) => UpdateFetcher<T, R>;

type UpsertFetcher<T extends GenericTable, R> = (input: T['Insert'][]) => Promise<MutationFetcherResponse<R>[] | null>;
declare const buildUpsertFetcher: <S extends GenericSchema$1, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, opts: LoadQueryOps<Q>) => UpsertFetcher<T, R>;

export { DeleteFetcher, InsertFetcher, LoadQueryOps, MutationFetcherResponse, PostgrestPaginationFetcher, PostgrestPaginationKeyDecoder, UpdateFetcher, UpsertFetcher, buildDeleteFetcher, buildInsertFetcher, buildUpdateFetcher, buildUpsertFetcher, createPaginationFetcher, createPaginationHasMoreFetcher, fetcher, loadQuery };
